/*******************************************************************************
 * Copyright (c) 2006 Matthew Hall and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     Matthew Hall - initial API and implementation
 ******************************************************************************/
package formslayout;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;

/**
 * Widget wrapper for displaying a separator label.
 * 
 * @author Matthew
 */
public class SeparatorLabel extends Composite {
	Label label;
	Font font;

	/**
	 * Constructs a new Separator.
	 * 
	 * @param parent
	 * 		the parent composite.
	 * @param style
	 * 		the separator style
	 */
	public SeparatorLabel(Composite parent, int style) {
		super(parent, style);
		setLayout(new FormLayout("p, 4dlu, f:d:g", "c:p"));

		label = new Label(this, SWT.NONE);
		label.setFont(font = enlargeDefaultFont(label));
		label.setForeground(label.getDisplay().getSystemColor(
				SWT.COLOR_LIST_SELECTION));
		label.setLayoutData(FormData.xy(0, 0));
		new Label(this, SWT.SEPARATOR | SWT.HORIZONTAL).setLayoutData(FormData
				.xy(2, 0));
		addListener(SWT.Dispose, new Listener() {
			public void handleEvent(Event event) {
				handleDispose();
			}
		});
	}

	private static Font enlargeDefaultFont(Control control) {
		FontData data = control.getFont().getFontData()[0];
		data = new FontData(data.getName(), data.getHeight() * 5 / 4, data
				.getStyle()
				| SWT.BOLD);
		return new Font(control.getDisplay(), data);
	}

	/**
	 * Constructs a new Separator.
	 * 
	 * @param parent
	 * 		the parent composite.
	 * @param text
	 * 		the separator text.
	 */
	public SeparatorLabel(Composite parent, String text) {
		this(parent, SWT.NONE);
		setText(text);
	}

	/**
	 * Sets the separator text.
	 * 
	 * @param text
	 * 		the separator text.
	 */
	public void setText(String text) {
		checkWidget();

		if (text == null)
			SWT.error(SWT.ERROR_NULL_ARGUMENT);

		label.setText(text);
		layout(new Control[] { label });
	}

	/**
	 * Returns the separator text.
	 * 
	 * @return the separator text.
	 */
	public String getText() {
		checkWidget();

		return label.getText();
	}

	private void handleDispose() {
		label = null;
		if (font != null)
			font.dispose();
		font = null;
	}
}
